package legion.muyue.best2048;

import android.util.Log;
import java.util.ArrayList;
import java.util.Random;

public class Game {

    private int[][] gameBoard;
    private Random random;
    private int score = 0;

    public Game() {
        gameBoard = new int[4][4];
        random = new Random();
    }

    public void printArray() {
        for (int[] row : gameBoard) {
            String rowString = String.format("%6d%6d%6d%6d%n", row[0], row[1], row[2], row[3]);
            Log.d("Game", rowString);
        }
        Log.d("Game", "\n");
    }

    public void addNewNumbers() {
        ArrayList<int[]> emptySpaces = new ArrayList<>();

        for (int x = 0; x < 4; x++) {
            for (int y = 0; y < 4; y++) {
                if (gameBoard[x][y] == 0) {
                    emptySpaces.add(new int[]{x, y});
                }
            }
        }

        if (!emptySpaces.isEmpty()) {
            int choice = random.nextInt(emptySpaces.size());
            int[] coordinates = emptySpaces.get(choice);
            int x = coordinates[0];
            int y = coordinates[1];
            int newNumber;
            int randomNumber = random.nextInt(100);
            if (randomNumber < 85) {
                newNumber = 2;
            } else if (randomNumber < 95) {
                newNumber = 4;
            } else {
                newNumber = 8;
            }
            gameBoard[x][y] = newNumber;
        }
    }


    public void pushUp() {
        Log.d("Game", "Pushing Up"); // Pour le débogage
        boolean[] alreadyCombined = new boolean[4]; // Tableau pour éviter les doubles fusions par colonne

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4]; // Réinitialise pour chaque colonne
            for (int x = 1; x < 4; x++) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    while (currentX > 0 && gameBoard[currentX - 1][y] == 0) {
                        currentX--;
                    }

                    if (currentX == 0) {
                        gameBoard[0][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX - 1][y] != value) {
                        gameBoard[currentX][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX - 1]) {
                        gameBoard[currentX - 1][y] *= 2;
                        score += gameBoard[currentX - 1][y];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentX - 1] = true;
                    } else {
                        // La case cible a déjà fusionné, on place la tuile juste en dessous
                        gameBoard[currentX][y] = value;
                        // Vide la case d'origine seulement si la tuile a bougé
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }

    public void pushDown() {
        Log.d("Game", "Pushing Down"); // Pour le débogage
        boolean[] alreadyCombined = new boolean[4]; // Tableau pour éviter les doubles fusions par colonne

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4]; // Réinitialise pour chaque colonne
            for (int x = 2; x >= 0; x--) { // Commence à l'avant-dernière ligne et remonte
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    // Déplace vers le bas tant que la case en dessous est vide
                    while (currentX < 3 && gameBoard[currentX + 1][y] == 0) {
                        currentX++; // Incrémente pour descendre d'une ligne
                    }

                    if (currentX == 3) { // Arrivé tout en bas
                        gameBoard[3][y] = value;
                        if (currentX != x) // Vide seulement si déplacé
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX + 1][y] != value) { // Case en dessous différente
                        gameBoard[currentX][y] = value; // Place juste au-dessus
                        if (currentX != x) // Vide seulement si déplacé
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX + 1]) { // Case en dessous identique ET non fusionnée
                        gameBoard[currentX + 1][y] *= 2; // Fusionne
                        score += gameBoard[currentX + 1][y];
                        gameBoard[x][y] = 0; // Vide la case d'origine
                        alreadyCombined[currentX + 1] = true; // Marque comme fusionné
                    } else {
                        // La case cible a déjà fusionné, on place la tuile juste au-dessus
                        gameBoard[currentX][y] = value;
                        // Vide seulement si déplacé
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }
}